!********************************************************************************
!*	Electron-Phonon energy transfer rate					*
!*	based on abinit and Lin et al. PRB 2008					*
!*										*
!*	Jan Vorberger								*
!*	Max Planck Institute for the Physics of complex systems			*
!*	18.02.2013								*
!********************************************************************************
!-ffree-line-length-none

!********************************************************************************
!*	Modules									*
!********************************************************************************
	module fields
	implicit none
	 integer fld,fla
	 real*8 ef,dosfake,dosreal,dosrealt,muf,muft
	 real*8 kb,pi,Te,Tp,hbar
	 real*8, dimension(:), allocatable :: wa2f,a2f,wdos,dos
!	 [kb]=ryd,[hbar]=Ha s
	 parameter (kb=0.633376d-5,pi=3.141592654d0,hbar=2.42d-17)
	end module fields
	
!********************************************************************************
!*	MMAAIINN								*
!********************************************************************************

	program el_ph

	use fields

	implicit none
	integer l,nd,na,nt,count
	integer j,grad
	parameter (nd=15,na=10)
	real*8 a,b,eps,cell,ne
	real*8 df,af1,af2,Te1,Te0,dTe
	real*8 rgauss,dosint,a2fint1,a2fint2
	real*8 ddosreal,z1,z2,ddosrealt,cve,dfd,dfu,dmu1,dmu2
	character*140 a2fname,dosname

	external df,af1,af2,dfd,dfu,dmu1,dmu2

!*******input********************************************************************
	write(*,*) 'Electron-Phonon Rate'
	write(*,*) '===================='
	write(*,*)
!	write(*,*) 'alpha^2F(w) file'
!	read(*,*) a2fname
	a2fname='co_elphon_4.out_ep_A2F_1'
!	a2fname='co_elphon_4.out_ep_A2F_2'
!	write(*,*) 'DOS file'
!	read(*,*) dosname
	dosname='co_elphon_1o_DS1_DOS_su_no_core'
!	dosname='co_elphon_1o_DS1_DOS_sd_no_core'	
	write(*,*) 'Electron Temperature start [K]'
!	read(*,*) Te0
	Te0=300.
	write(*,*) 'Electron Temperature end [K]'
!	read(*,*) Te1
	Te1=10000
	write(*,*) 'Electron Temperature points [K]'
!	read(*,*) nt
	nt=400
!	write(*,*) 'Lattice Temperature [K]'
!	read(*,*) Tp
	Tp=300.
!	write(*,*) 'Electron Number:'
!	read(*,*) ne
	ne=18.63-8.
!	ne=15.34-8.
!	write(*,*) 'Fermi energy [Ha] (all energy units are Ha):'
!	read(*,*) ef
	ef=0.4379
!	write(*,*) 'DOS @ Fermi level from A2F file:'
!	read(*,*) dosfake
!there is a factor of two between the value in a2f file and DOS file for Al
!	T=0
	dosfake=8.3
!	dosfake=40.0
!	 write(*,*) 'cell volume [A^3]'
!	 read(*,*) cell
	 cell=22.152

	dTe=(Te1-Te0)/dble(nt)

	l=len_trim(a2fname)
	a2fname='L:/9_hzdr/dft/co/elphon/' //a2fname(1:l)
!
	l=len_trim(dosname)
	dosname='L:/9_hzdr/dft/co/elphon/' // dosname(1:l)

!*******calculations*************************************************************
!	write(*,*) a2fname
	call get_length(a2fname,fla)
	fla=fla-na
	allocate(wa2f(1:fla),a2f(1:fla))
!	write(*,*) fla

!	write(*,*) dosname
	call get_length(dosname,fld)
	fld=fld-nd
	allocate(wdos(1:fld),dos(1:fld))
!	write(*,*) fld

	call get_field(a2fname,a2f,wa2f,fla,na)
	call get_field(dosname,dos,wdos,fld,nd)

!-------DOS, Fermi energy, chemical potential etc.
	call locate(wdos,fld,ef,j)
	grad=2
	l=min(max(j-(grad-1)/2,1),fld+1-grad)
	call polint(wdos(l),dos(l),grad,ef,dosreal,ddosreal)
	write(*,*) 'E_F [Ha]', ef
	write(*,*) 'DOS at E_F, from a2F:',dosfake
	write(*,*) 'DOS at E_F, from DOS, "T=0":',dosreal

	do count=1,nt

	 Te=Te0+(count-1)*dTe
!--------get chemical potential from conservation of electron number
	 call get_chem_pot(ne,muf)
!	 write(*,*) 'mu_F[Ha]',muf
	 call locate(wdos,fld,muf,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,muf,dosrealt,ddosrealt)
!	 write(*,*) 'DOS at mu_F, from DOS, "T>0":',dosrealt

!--------integrate over DOS and df/de
	 eps=1.d-5
	 a=wdos(1)
	 b=wdos(fld)
	 dosint=rgauss(df,a,b,eps)
!	 write(*,*) 'Dos int=',dosint

!-------get electron heat capacity
	 eps=1.d-5
	 a=wdos(1)
	 b=wdos(fld)
	 !get dmu/dT
	 muft=rgauss(dmu1,a,b,eps)
	 muft=muft/rgauss(dmu2,a,b,eps)
	 muft=-muft/Te
	 cve=rgauss(dfd,a,b,eps)

!--------get lambda<w^2>
	 a=wa2f(1)
	 b=wa2f(fla)
	 !full bose function
	 a2fint1=rgauss(af1,a,b,eps)
	 !bose function expanded
	 a2fint2=rgauss(af2,a,b,eps)
!	 write(*,*) 'lambda<w^2>[Ha^2]=',2.d0*a2fint2
!	 write(*,*) 'lambda<w^2>[meV^2]=',2.d0*a2fint2*4.d0*13.6*13.6*1.d6

!--------get electron-phonon coupling
!	 write(*,*) '[Ha/cell/K/s]'
	 z1=pi/hbar*kb/2.d0*2.d0*a2fint2*dosint/dosrealt
!	 write(*,*) 'Z/(T_e-T_l)=\lambda<w^2>*DOSint=',z1
	 z2=pi/hbar*2.d0*a2fint1*dosint/dosrealt/(Te-Tp)
!	 write(*,*) 'Z/(T_e-T_l)=int w^2*a^2F(w)*(nbp-nbe)*DOSint=',z2


!	 write(*,*) '[W/m^3/K]'
	 z1=z1*4.357d-18/cell/1.d-30
!	 write(*,*) 'Z/(T_e-T_l)=\lambda<w^2>*DOSint=',z1
	 z2=z2*4.357d-18/cell/1.d-30
!	 write(*,*) 'Z/(T_e-T_l)=int w^2*a^2F(w)*(nbp-nbe)*DOSint=',z2
!	 write(*,*)
!	 heat capacity unit into J/m^3/K
	 cve=cve*2*13.6*1.602e-19/cell/1.e-30
	 write(*,'(f9.2,2Es15.7,2f11.6,Es15.7)') Te,z1,z2,muf,dosrealt,cve
	end do
!*******end of main**************************************************************
	deallocate(wa2f,a2f,wdos,dos)
	end

!********************************************************************************
!*	Subroutines								*
!********************************************************************************

!********************************************************************************
!*	get chemical potential at T>0						*
!********************************************************************************

	subroutine get_chem_pot(ne,mu)
	use fields
	implicit none
	real*8 mu,a,b,eps,mue,x1,x2,n,ne
	real*8 funk,rtbis
	common /ele/ n
	external funk

	 n=dble(ne)
	 eps=1.0d-8
	 x1=wdos(1)
	 x2=wdos(fld)
	 mu=rtbis(funk,x1,x2,eps)

	return
	end
!-----------------------
	real*8 function funk(mu)
	use fields
	implicit none
	real*8 mu,mue,rgauss,intdosf,a,b,eps,n,trapez
	common /ele/ n
	common /chem/ mue
	external intdosf

	 mue=mu
	 eps=1.0d-7
	 a=wdos(1)
	 b=wdos(fld)
!	 funk=n-rgauss(intdosf,a,b,eps)
	 funk=n-trapez(intdosf,a,b,fld)
!	 write(*,*) mue,funk,n,trapez(intdosf,a,b,fld)
	return
	end
!-----------------------
	real*8 function intdosf(e)
	use fields
	implicit none
	integer j,l,grad
	real*8 e,f,dosate,ddosate,mue
	common /chem/ mue

	 call locate(wdos,fld,e,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,e,dosate,ddosate)
	 f=1.d0/(dexp((e-mue)*2.d0/kb/Te)+1.d0)
	 intdosf=f*dosate
!	 write(*,*) e,f,dosate
	return
	end

!********************************************************************************
!*	Integration								*
!********************************************************************************

!int over DOS w and df/dt for heat capacity--------------------------------------
	real*8 function dfd(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,dosatw,ddosatw,f1,expo

	 call locate(wdos,fld,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,w,dosatw,ddosatw)
	 expo=(w-muf)*2.d0/kb/Te
	 if (expo.gt.20.d0) then
	  f1=dexp(-expo)*(expo/Te+muft*2.d0/kb/Te)
	 else
	  f1=1.d0/(dexp(expo)+1.d0)
	  f1=f1*f1*dexp(expo)*(expo/Te+muft*2.d0/kb/Te)
	 endif
	 dfd=dosatw*w*f1
!	write(17,'(4Es17.7E3)') w,dfd,dosatw,f1

	return
	end

!int over DOS and df/dT for dmu/dT------------------------
	real*8 function dmu1(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,dosatw,ddosatw,f1,expo

	 call locate(wdos,fld,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,w,dosatw,ddosatw)
	 expo=(w-muf)*2.d0/kb/Te
	 if (expo.gt.20.d0) then
	  f1=dexp(-expo)
	 else
	  f1=1.d0/(dexp(expo)+1.d0)
	  f1=f1*f1*dexp(expo)
	 endif
	 dmu1=dosatw*(w-muf)*f1

	return
	end



	real*8 function dmu2(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,dosatw,ddosatw,f1,expo

	 call locate(wdos,fld,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,w,dosatw,ddosatw)
	 expo=(w-muf)*2.d0/kb/Te
	 if (expo.gt.20.d0) then
	  f1=dexp(-expo)
	 else
	  f1=1.d0/(dexp(expo)+1.d0)
	  f1=f1*f1*dexp(expo)
	 endif
	 dmu2=dosatw*f1

	return
	end

!int over DOS^2 and df/de for e-ph coupling----------------------------
	real*8 function df(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,dosatw,ddosatw,f1,expo

	 call locate(wdos,fld,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fld+1-grad)
	 call polint(wdos(l),dos(l),grad,w,dosatw,ddosatw)
	 expo=(w-muf)*2.d0/kb/Te
	 if (expo.gt.20.d0) then
	  f1=dexp(-expo)*2.d0/kb/Te
	 else
	  f1=1.d0/(dexp(expo)+1.d0)
	  f1=f1*f1*dexp(expo)*2.d0/kb/Te
	 endif
	 df=dosatw*dosatw*f1
!	write(18,'(4Es15.7)') w,df,dosatw,f1
	return
	end

!int over w^2*a^2F*(nbp-nbe) for e-ph coupling----------------------------
	real*8 function af1(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,a2fatw,da2fatw,f1
	real*8 nbp,nbe

	 call locate(wa2f,fla,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fla+1-grad)
	 call polint(wa2f(l),a2f(l),grad,w,a2fatw,da2fatw)
	 nbe=1.d0/(dexp(w*2.d0/kb/Te)-1.d0)
	 nbp=1.d0/(dexp(w*2.d0/kb/Tp)-1.d0)
	 af1=w*w*a2fatw*(nbe-nbp)
!	write(19,'(4Es15.7)') w,af1,a2fatw,nbe-nbp
	return
	end

!int over w*a^2F for e-ph coupling----------------------------
	real*8 function af2(w)

	use fields

	implicit none
	integer grad,j,l
	real*8 w,a2fatw,da2fatw,f1

	 call locate(wa2f,fla,w,j)
	 grad=2
	 l=min(max(j-(grad-1)/2,1),fla+1-grad)
	 call polint(wa2f(l),a2f(l),grad,w,a2fatw,da2fatw)
	 af2=w*a2fatw
!	write(20,'(4Es15.7)') w,af2,a2fatw,kb*(Te/w-Tp/w)
	return
	end

!********************************************************************************
!*	gets field from file							*
!********************************************************************************

	subroutine get_field(filename,field,wfield,length,nx)
	implicit none
	integer length,t,nx
	real*8 field(1:length),wfield(1:length)
	character*10 dummy
	character*140 filename

	open(13,file=filename,status='old')
	do t=1,nx
	 read(13,*) dummy
	end do
	do t=1,length
	 read(13,*) wfield(t),field(t)
	end do
	close(13)
	return
	end

!********************************************************************************
!*	determines field length							*
!********************************************************************************

	subroutine get_length(filename,m)
	implicit none
	integer m,io
	character*10 dummy
	character*140 filename

	 m=0
	 open(13,file=filename,status='old')
	 do while (.true.)
	  read(13,*,iostat=io) dummy
	  m=m+1
	 if (io.lt.0) EXIT
	 end do
	 close(13)
	 m=m-1
	return
	end

!*******************************************************************************
      SUBROUTINE locate(xx,n,x,j)
      INTEGER j,n
      REAL*8 x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).gt.xx(1)).eqv.(x.gt.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      j=jl
      return
      END
!  (C) Copr. 1986-92 Numerical Recipes Software 5.)'j)&3|,$.

      SUBROUTINE polint(xa,ya,n,x,y,dy)
      INTEGER n,NMAX
      REAL*8 dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=10)
      INTEGER i,m,ns
      REAL*8 den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.)stop 'failure in polint'
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
!  (C) Copr. 1986-92 Numerical Recipes Software 5.)'j)&3|,$.

!-----------------------------------------------------------------------

	real*8 function trapez(f,a,b,n)
	
	implicit none
	integer n,i
	real*8 f,a,b,eps,dx,x
	external f
	
	dx=(b-a)/dble(n)
	trapez=0.d0
	do i=1,n
	 x=a+dx*dble(i-1)
	 trapez=trapez+(f(x)+f(x+dx))*0.5d0
	end do
	
	trapez=trapez*dx
	
	return
	end

!-----------------------------------------------------------------------

      real*8 function rgauss(f, a, b, eps)
!
!     **********************************************************
!
!     ADAPTIVE REAL GAUSSIAN QUADRATURE
!
!     dgauss is set equal to the approximate value of the
!     integral of the function f over the interval (a,b),
!     with accuracy parameter eps
!     AUTHOR: K.S. Koelbig (CERN)
!     **********************************************************
!
      real*8 f, a, b, eps
      real*8 w(12), x(12), aa, bb, c1, c2, u, s8, s16, const
      external         f

      data w / 0.1012285362903762591525313543E00, 0.2223810344533744705443559944E00, 0.3137066458778872873379622020E00, 0.3626837833783619829651504493E00, 0.2715245941175409485178057246E-1, 0.6225352393864789286284383699E-1, 0.9515851168249278480992510760E-1, 0.1246289712555338720524762822E00, 0.1495959888165767320815017305E00, 0.1691565193950025381893120790E00, 0.1826034150449235888667636680E00, 0.1894506104550684962853967232E00/

      data x / 0.9602898564975362316835608686E00, 0.7966664774136267395915539365E00, 0.5255324099163289858177390492E00, 0.1834346424956498049394761424E00, 0.9894009349916499325961541753E00, 0.9445750230732325760779884155E00, 0.8656312023878317438804678977E00, 0.7554044083550030338951011948E00, 0.6178762444026437484466717640E00, 0.4580167776572273863424194430E00, 0.2816035507792589132304605015E00, 0.9501250983763744018531933543E-1/

!  START
      rgauss = 0.0E0
      if (b .eq. a) return
      const  = 1.d-10/(b-a)
      bb     = a

!  COMPUTATIONAL LOOP
1     aa = bb
      bb = b
2          c1 = 0.5E0*(bb+aa)
           c2 = 0.5E0*(bb-aa)
           s8 = 0.0E0
           do 3, i=1,4
             u  = c2*x(i)
             s8 = s8 + w(i)*(f(c1+u)+f(c1-u))
3          continue
           s8  = c2*s8
           s16 = 0.0E0
           do 4 i=5,12
             u   = c2*x(i)
             s16 = s16 + w(i)*(f(c1+u)+f(c1-u))
4          continue
           s16 = c2*s16
! Originalcheck
!           if ( abs(s16-s8) .le. eps*(1.E0+abs(s16)) ) go to 5
! Mein Check auf Genauigkeit:
		if ( abs(s16-s8) .le. eps/2.E0*abs(s16+s8)) go to 5
           bb = c1
	     if (2.E0*c2 .ge. eps*32.E0*const) go to 2
!      write(*,6)
5     rgauss = rgauss + s16
      if (bb .ne. b) go to 1
      return
6     format(4X, 'FUNCTION RGAUSS ... TOO HIGH ACCURACY REQUIRED')
      end

!********************************************************************************
!*	bisection								*
!********************************************************************************

      real*8 FUNCTION rtbis(func,x1,x2,xacc)
      INTEGER JMAX
      REAL*8 x1,x2,xacc,func
      EXTERNAL func
      PARAMETER (JMAX=40)
      INTEGER j
      REAL*8 dx,f,fmid,xmid
      fmid=func(x2)
      f=func(x1)
      if(f*fmid.ge.0.) stop 'root must be bracketed in rtbis'
      if(f.lt.0.)then
        rtbis=x1
        dx=x2-x1
      else
        rtbis=x2
        dx=x1-x2
      endif
      do 11 j=1,JMAX
        dx=dx*.5
        xmid=rtbis+dx
        fmid=func(xmid)
        if(fmid.le.0.)rtbis=xmid
        if(abs(dx).lt.xacc .or. fmid.eq.0.) return
11    continue
      stop 'too many bisections in rtbis'
      END
!  (C) Copr. 1986-92 Numerical Recipes Software 5.)'j)&3|,$.
